# ABOSS, filtering Ig-seq data with structural information

ABOSS is a structurally motivated software to filter antibody sequences that might not fold into a viable protein. As such it is an error correction protocols on for Ig-seq outputs.

## Prerequisites for the Installation.

You should have HMMER installed so that we can number your sequences. We tested the code with hmmer version 3.1b:

```
http://eddylab.org/software/hmmer3/3.1b2/hmmer-3.1b2.tar.gz
```

All the python dependencies are listed in the REQUIREMENTS file that can be installed via:

```
sudo pip install -r REQUIREMENTS
```

It is adviseable to install the requirements in a separate virtual environment.

If during the requirements install you encounter a problem with existing 'six' install, issuing this 

```
sudo pip install six --ignore-installed six
```

and run again:

```
sudo pip install -r REQUIREMENTS
```
Depending on your system, you might have to install tkinter manually so that matplotlib works. For instance on ubuntu system this can be done by:

```
sudo apt-get install python-tk
```

##Quickstart.

If you simply want to check if the program runs correctly on your system, issue the following command from the home directory of this code.

```
./run_aboss -f test_ig-seq_dataset.pkl -c H -r 10000 -s human -n 8
```

If the program finished successfully, it points to a successful setup and you can read on below to understand arguments, parameters, input and output of the program.

## This is the ABOSS manual that will help you to run ABOSS on your machine.

ABOSS incorporates following files: run_aboss,run_ANARCI_parsing.py, reference_matrix.py, IMGT_germline_reference.py, plotting.py, redundancy_plot.py, flagging_residue_position.py ,run_fasta.py, write_to_csv.py, germlines.pckl as well as ANARCI package.

ABOSS takes the input of amino acid sequences in the **FASTA format** or python dictionary in the **pickle format**, where the key is the amino acid sequnce and its value is redundancy.

ABOSS comes with a test file **test_ig-seq_dataset.pkl**, where Ig-seq data is saved in python dictionary. This dataset is from Galson et al., (2016) study.

For example:

	{"QVQLQQSGAEVVRSGASVKLSCTASGFNIKDYYIHWVKQRPEKGLEWIGWIDPEIGDTEYVPKFQGKATMTADTSSNTAYLQLSSLTSEDTAVYYCNAGHDYAARGRFPYWGQGTLVTVSA": 3}

Stages of ABOSS (in sequencial order):

	1) run_fasta.py (Optional) - Conversion of the **FASTA** file to the **python dictionary** where the key is an antibody sequence and corresponding redundancy.

	2) run_ANARCI_parsing.py - ANARCI parcing.

	3) IMGT_germline_reference.py - Generation of the IMGT Germline reference.

	4) reference_matrix.py - Generation of the positional amino acid reference matrix and calculating the structutal viability error rate (Saved in Error/ig_seq_residue_error_rate.txt).

	5) flagging_residue_position.py - Running ANARCI parsed sequences through the reference matrix.

	6) plotting.py - Plotting graphs of positional amino acid proportions, Shannon entropy and the number of entries for each of the antibody region.

	7) redundancy_plot.py - We plot the graph of the number of incorrect residues in an individual sequence against its redundancy.

	8) write_to_csv.py - Conversion of ABOSS outputs to .csv format. The CSV file contains ANARCI parsed sequences, Redundancy, CDR-H3, Number of flagged residue/positions, Incorrect residue/positions,  V and J genes, and ANARCI numbered sequences.

## COMMAND LINE OPTIONS

#### To run ABOSS:
	
	run_aboss -f [INPUT FILE]
	
	"-h",  <Help message>	
	
	"-f",  <FASTA or PICKLE file of python dictionary>
	
	"-a",  <True if the input file is in the **FASTA** format, default is False>
	
	"-c",  <Indicate the variable chain [H or L] of antibodies in the Ig-seq data. H is default>

	"-r",  <Indicate the rate of ABOSS analysis, e.g. how many sequences are parsed with ANARCI at a time. Default rate is 10000>

	"-s",  <Indicate the species [human, mouse, rhesus, rabbit, alpaca] of the Ig-seq data. The default species is human>

	"-n",  <The number of cores engaged in ABOSS analysis. The default number of cores is 8>

## Some examples how to run ABOSS:

The ABOSS analysis of a FASTA file from human V-chain Ig-seq data on a 8-core desktop machine:

	./run_aboss -f YOUR_file.fa -a True -c H -r 10000 -s human -n 8

The ABOSS analysis of a FASTA file from human V-chain Ig-seq on a 44-core server:

	./run_aboss -f YOUR_file.fa -a True -c H -r 40000 -s human -n 44

The ABOSS analysis of a pickle file from human V-chain Ig-seq on a 8-core desktop machine:

	./run_aboss -f YOUR_file.pkl -c H -r 10000 -s human -n 8

##  ABOSS output files

	"Final_Ig-seq_information.csv" - <The CSV file that contains ANARCI Parsed sequences, Redundancy, CDR-H3, Number of flagged residue/positions, Flagged residue/positions (None is no flagged positions), and V and J genes, ANARCI numbered sequences >

	"YOUR_filtered_sequenced.pkl" - <The ABOSS output file is a python dictionary in the pickle format. The first key is the number of flagged residue/positions and the value is another dictionary, where the key is the antibody sequence and the value is the corresponding sequence redundancy>

	"YOUR_filtered_amino_acid_by_position.pkl" - <A python dictionary that holds the number of residues at antibody positions in Ig-seq data. The data is also separated by the number of flagged residue/positions>

	"Graphs/" - <The folder that contains figures generated in ABOSS analysis. These plots include positional amino acid distributions, Shannon entropy and number of entries in Ig-seq sequences with zero flagged residue/positions as well as the plot of the number of flags in an individual sequence against its redundancy.>

	"Error/ig_seq_residue_error_rate.txt" - <The text file with the calculated residue error rate for your Ig-seq data.>

