import pandas as pd
import pickle
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import math
import os
from os.path import join
import sys

def new_calc_pos_length(frame_aa):
	"""
	Here we calculate the number of entries by position
	"""
	pos_length = {}
	for pos in frame_aa:
		for ele in frame_aa[pos]:
			if pos not in pos_length:
				pos_length[pos] = 0
			pos_length[pos] += frame_aa[pos][ele]
	return pos_length

def control_bar(frame, counting_aa, pos_length):
	"""
	Here we create control bars for entropy and amino acid distributions
	by antibody region.
	"""
	length_of_frame = 0
	entropy = 0
	entropy_contr = {frame:{}}
	control_frame_prop = {frame:{}}
	weigthed_contr = {frame:{}}
	for pos in pos_length:
		length_of_frame += pos_length[pos]

	for pos in counting_aa:
		for ele in counting_aa[pos]:
			if ele not in control_frame_prop[frame].keys():
				control_frame_prop[frame][ele] = counting_aa[pos][ele]
			else:
				control_frame_prop[frame][ele] += counting_aa[pos][ele]
	for ele in control_frame_prop[frame]:
		weigthed_contr[frame][ele] = control_frame_prop[frame][ele] / float(length_of_frame)
		if weigthed_contr[frame][ele] > 0:
           		entropy += - weigthed_contr[frame][ele]*math.log(weigthed_contr[frame][ele], 2)
	entropy_contr[frame] = entropy
	return weigthed_contr, entropy_contr

def adding_zeros(dictionary):
	amino_acids = sorted(list("QWERTYIPASDFGHKLCVNM"))
	for pos in dictionary:
		for amino in amino_acids:
			if amino not in dictionary[pos]:
				dictionary[pos][amino] = 0
	return dictionary

def frame_canon(cols):
	"""
	Making sure that IMGT positions are sorted
	"""
	integer_list =  [int(x) for x in cols]
	sorted_integer_list = sorted(integer_list,key=int)
	sorted_string_list = [str(x) for x in sorted_integer_list]
	return sorted_string_list

def cdrh3(cols):
	"""
	Making sure that cdr3 IMGT positions are sorted
	"""
	integer_list = []
	pos_111 = []
	pos_112 = []
	sorted_string_list = []
	for pos in cols:
		try:
			integer_list.append(int(pos))
			sorted_integer_list = sorted(integer_list,key=int)
			sorted_string_list = [str(x) for x in sorted_integer_list]
		except:
			if "111" in pos:
				pos_111.append(str(pos))
			elif "112" in pos:
				pos_112.append(str(pos))
	if "112A" in pos_112:
		pos_111_112_list = sorted(pos_111) + sorted(pos_112)[::-1]
		sorted_string_list = sorted_string_list[:sorted_string_list.index("111")+1] + pos_111_112_list + sorted_string_list[sorted_string_list.index("111")+1:]
	return sorted_string_list	

def all_aa_proportions():
	error = 0
	frame = sys.argv[1]
	entropy_dict = {}
	if frame not in entropy_dict:
		entropy_dict[frame] = {}
	#################################
	# Here we create a Graph folder #
	#################################
	cwd = os.getcwd()
	if os.path.isdir(join(cwd, "Graphs")) != True:
		os.makedirs("Graphs")
	with open("counting_aa_polished.pkl","rb") as q:  # Dictionary that contains amino acid distribution by position
		counting_aa = pickle.load(q)[0][frame]	  # We analyze one antibody region here
	counting_aa = adding_zeros(counting_aa)
	pos_length = new_calc_pos_length(counting_aa)
	weighted_contr, entropy_contr = control_bar(frame, counting_aa, pos_length)
	##############################################
	### Here we calculate entropy by position ####
	##############################################
	for pos in counting_aa:
		entropy = 0
		for ele in counting_aa[pos]:
			 counting_aa[pos][ele] = float(counting_aa[pos][ele]) / pos_length[pos]  # Amino acid distribution by position
			 if counting_aa[pos][ele] > 0:
   				 entropy += -counting_aa[pos][ele]*math.log(counting_aa[pos][ele], 2) # Entropy by position
		entropy_dict[frame][pos] = entropy
	#########################################################
	if weighted_contr == {frame: {}}:
		print "The input files didn't contain sequence of ", frame
	else:	
		df_entropy = pd.DataFrame(entropy_dict[frame], index = [0]).fillna(0)  # Dataframe of position entropies
		df_aa_proportion = pd.DataFrame(counting_aa).fillna(0)		       # Dataframe of amino acid distribution by position
		df_pos_len = pd.DataFrame(pos_length, index=[0]).fillna(0) 	       # Dataframe of Number of entries by position
		########################################################
		##### Here we put amino acid in the numberic order #####
		########################################################
		cols = df_aa_proportion.columns.tolist()
		if frame not in ["cdrh3","cdrl3"]:
			print frame
			new_cols = frame_canon(cols)
		else:
			new_cols = cdrh3(cols)	
		print new_cols
		###############################
		##### ENTROPY PLOTTING  #######
		###############################
		df_entropy = df_entropy[new_cols]
		df_entropy = df_entropy.transpose()
		entropy_control_bar = pd.DataFrame(entropy_contr, index = [0]).fillna(0)  # entropy_control
		entropy_control_bar = entropy_control_bar.transpose()
		entropy_complete_plot = df_entropy.append(entropy_control_bar)
		########################################
		### Graph for entries at positions #####
		########################################
		df_pos_len = df_pos_len[new_cols]
		df_pos_len = df_pos_len.transpose()
		###########################################
		##### AA distribution by position plot ####
		###########################################
		df_aa_proportion = df_aa_proportion[new_cols]
		df_aa_proportion = df_aa_proportion.transpose()
		averaged_aa_control_bar = pd.DataFrame(weighted_contr).fillna(0) 
		averaged_aa_control_bar = averaged_aa_control_bar.transpose()	
		aa_proportion_complete_plot = df_aa_proportion.append(averaged_aa_control_bar)
		###### Plotting starts here########
		### entropy by position ###
		fig, axis = plt.subplots()
		entropy_complete_plot.plot.bar(title ="%s entropy, ABOSS filtered data" %(frame), legend = False, color="red")
		fig.set_size_inches(7.5,4.5, forward=True)
		plt.savefig("Graphs/%s entropy ABOSS filtered data" %(frame), dpi= 300)
		plt.clf()
		### amino acids by position ###
		colors_dict = dict([("A", "#a2a0a5"),("C", "#fff319"),("D","#22a56f"),("E","#19ff34"),("F","#031dc1"),("G","#35ffee"),("H","#6e728c"),("I","#057200"), ("K","#720070"),("L","#914b00"),("M","#847806"),("N","#f24d76"),("P","#fcc7ed"),("Q","#cc0c0c"),("R","#2cc124"),("S","#c124c1"),("T","#ff6600"),("V","#c6e833"),("W","#ff0000"),("Y","#6fa522")])
		amino_acid_order =  list(aa_proportion_complete_plot)
		my_colors = [colors_dict[x] for x in amino_acid_order]
		fig, axis = plt.subplots()
		aa_proportion_complete_plot.plot.bar(stacked=True, color = my_colors, title =" %s proportions, ABOSS filtered data" %(frame), legend='reverse').legend(bbox_to_anchor=(0.99, 1.02))
		fig.set_size_inches(9,4.5, forward=True)
		plt.savefig("Graphs/%s proportion ABOSS filtered data" %(frame),dpi= 300)
		plt.clf()
		### entries by postion ###
		fig, axis = plt.subplots()
		df_pos_len.plot.bar(color="green", legend = False, title=" %s entries, ABOSS filtered data" %(frame))
		fig.set_size_inches(7.5,4.5, forward=True)
		plt.savefig("Graphs/%s entries ABOSS filtered data" %(frame), dpi= 300)
		plt.clf()
		return "The analysis of %s domain has finished" % (frame)

if __name__ == '__main__':
	all_aa_proportions()
