import pandas as pd
import pickle
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import math
import os
from os.path import join
import sys
from optparse import OptionParser

def new_calc_pos_length(aa_one_CDR):
	pos_length = {}
	for pos in aa_one_CDR:
		for ele in aa_one_CDR[pos]:
			if pos not in pos_length:
				pos_length[pos] = 0
			pos_length[pos] += aa_one_CDR[pos][ele]
	return pos_length

def control_bar(frame, counting_aa, pos_length):
	length_of_frame = 0
	entropy = 0
	entropy_contr = {frame:{}}
	control_frame_prop = {frame:{}}
	weigthed_contr = {frame:{}}
	for pos in pos_length:
		length_of_frame += pos_length[pos]
	for pos in counting_aa:
		for ele in counting_aa[pos]:
			if ele not in control_frame_prop[frame].keys():
				control_frame_prop[frame][ele] = counting_aa[pos][ele]
			else:
				control_frame_prop[frame][ele] += counting_aa[pos][ele]
	for ele in control_frame_prop[frame]:
		weigthed_contr[frame][ele] = control_frame_prop[frame][ele] / float(length_of_frame)
#		m = weigthed_contr[frame][ele]
		if weigthed_contr[frame][ele] > 0:
           		entropy += - weigthed_contr[frame][ele]*math.log(weigthed_contr[frame][ele], 2)
	entropy_contr[frame] = entropy
	return weigthed_contr, entropy_contr

def noise_removal(pos_amino_acids, pos_length, framework, chain):
	"""
	This function 1) calculates the residue error rate
		      2) uses this residue error rate to create
	a reference matrix for each antibody region
	"""
	bad_aminos = set("BJXZ*")
	with open("IMGT_germlines.pkl", "rb") as cx: #insert IMGT germline reference dictionary
		IMGT_refence = pickle.load(cx)
	with open("counting_aa_raw.pkl","rb") as q:  # Insert dictionary here that contain amino acids by postition
		error_counting_aa = pickle.load(q)
	if chain == "H":
		error_fw1 = sorted(error_counting_aa["fwh1"]["23"].values(), key=lambda x: x, reverse=True)
		error_fw1 = [float(x) / sum(error_fw1) for x in error_fw1]
		error_fw3 = sorted(error_counting_aa["fwh3"]["104"].values(), key=lambda x: x, reverse=True)	
		error_fw3 = [float(x) / sum(error_fw3) for x in error_fw3]
	elif chain == "L":
		error_fw1 = sorted(error_counting_aa["fwl1"]["23"].values(), key=lambda x: x, reverse=True)
		error_fw1 = [float(x) / sum(error_fw1) for x in error_fw1]
		error_fw3 = sorted(error_counting_aa["fwl3"]["104"].values(), key=lambda x: x, reverse=True)	
		error_fw3 = [float(x) / sum(error_fw3) for x in error_fw3]
	if len(error_fw1) == 1 and len(error_fw3) == 1 :
		error = 0
	else:
		if len(error_fw1) > 1 and len(error_fw3) > 1:		
			if error_fw1[1] >= error_fw3[1]:
				error = error_fw1[1]
			else:
				error = error_fw3[1]
		elif len(error_fw1) == 1:
			error = error_fw3[1]
		else:
			error = error_fw1[1]
	amino_acid_by_pos = {}
	for position in pos_amino_acids:
		if position not in amino_acid_by_pos:
			amino_acid_by_pos[position] = {}
		for amino in pos_amino_acids[position]:
			if amino in bad_aminos:
				continue
			else:
				amino_pos_proportion = float(pos_amino_acids[position][amino]) / pos_length[position]

				if position in IMGT_refence[framework]:
					if IMGT_refence[framework][position][amino] == 0:
						if amino_pos_proportion <= error:
							amino_acid_by_pos[position][amino] = 0
						else:
							amino_acid_by_pos[position][amino] = pos_amino_acids[position][amino]
					else:
						amino_acid_by_pos[position][amino] = pos_amino_acids[position][amino]
				else:
					if amino_pos_proportion <= error:
						 amino_acid_by_pos[position][amino] = 0
					else:
						amino_acid_by_pos[position][amino] = pos_amino_acids[position][amino]
	return amino_acid_by_pos,error
			
def frame_canon(cols):
	integer_list =  [int(x) for x in cols]
	sorted_integer_list = sorted(integer_list,key=int)
	sorted_string_list = [str(x) for x in sorted_integer_list]
	return sorted_string_list

def cdrh3(cols):
	integer_list = []
	pos_111 = []
	pos_112 = []
	sorted_string_list = []
	for pos in cols:
		try:
			integer_list.append(int(pos))
			sorted_integer_list = sorted(integer_list,key=int)
			sorted_string_list = [str(x) for x in sorted_integer_list]
		except:
			if "111" in pos:
				pos_111.append(pos)
			elif "112" in pos:
				pos_112.append(pos)
	if "112A" in pos_112:
		pos_111_112_list = sorted(pos_111) + sorted(pos_112)[::-1]
		sorted_string_list = sorted_string_list[:sorted_string_list.index("111")+1] + pos_111_112_list + sorted_string_list[sorted_string_list.index("111")+1:]
	return sorted_string_list					

def all_aa_proportions(chain):
	frame = sys.argv[1]
	entropy_dict = {}
	if frame not in entropy_dict:
		entropy_dict[frame] = {}
	# Here we create two folders Reference_matrix and Error
	cwd = os.getcwd()
	if os.path.isdir(join(cwd, "Reference_matrix")) != True:
		os.makedirs("Reference_matrix")
	if os.path.isdir(join(cwd, "Error")) != True:
		os.makedirs("Error")
	with open("counting_aa_raw.pkl","rb") as q:  
		counting_aa_old = pickle.load(q)[frame]
	pos_length = new_calc_pos_length(counting_aa_old)
	counting_aa, residue_error_rate = noise_removal(counting_aa_old, pos_length, frame, chain) 
	pos_length = new_calc_pos_length(counting_aa)  # Here we recalculate number of entries per position after the removal of residue/positions found below the residue error rate
	weighted_contr, entropy_contr = control_bar(frame, counting_aa, pos_length)   # Here we get the control bars 
	###############################################
	## Here we calculate entropy by position ######
	###############################################	
	for pos in counting_aa:
		entropy = 0
		for ele in counting_aa[pos]:
			 counting_aa[pos][ele] = float(counting_aa[pos][ele]) / pos_length[pos]
			 
			 if counting_aa[pos][ele] > 0:
   				 entropy += - counting_aa[pos][ele]*math.log(counting_aa[pos][ele], 2)
		entropy_dict[frame][pos] = entropy
	##################################################
	if weighted_contr == {frame: {}}:
		print "The input files didn't contain sequence of ", frame
	else:
		df_entropy = pd.DataFrame(entropy_dict[frame], index = [0]).fillna(0)  # entropy_frame_specific
		df_aa_proportion = pd.DataFrame(counting_aa).fillna(0) # Amino acid Distribution Dataframe
		pos_hist = pd.DataFrame(pos_length, index = [0]).fillna(0) # This DataFrame is used to find positions with less than 20 entries
		df_pos_len = pd.DataFrame(pos_length, index=[0]).fillna(0) # Dataframe that holds numbers of entries by position
		########################################################
		##### Here we put amino acid in the numberic order #####
		########################################################
		cols = pos_hist.columns.tolist() # Column names
		for col in cols:
			if pos_hist[col][0] <= 20:  # Threshold for minimum entries at positions
				del df_aa_proportion[col]
				del df_entropy[col]
				del df_pos_len[col]
				print "Position with fewer than 20 entries: ", col
		cols = df_aa_proportion.columns.tolist()
		if frame not in ["cdrh3","cdrl3"]:
			new_cols = frame_canon(cols)
		else:
			new_cols = cdrh3(cols)		
		##########################
		## Position entry graph ##
		##########################
		df_pos_len = df_pos_len[new_cols]
		df_pos_len = df_pos_len.transpose()
		#################
		# Entropy graph #
		#################
		df_entropy = df_entropy[new_cols]	
		df_entropy = df_entropy.transpose()
		entropy_control_bar = pd.DataFrame(entropy_contr, index = [0]).fillna(0)  # entropy_control
		entropy_control_bar = entropy_control_bar.transpose()
		entropy_complete_plot = df_entropy.append(entropy_control_bar)
		##########################
		## AA proportion graph ###
		##########################		
		df_aa_proportion = df_aa_proportion[new_cols]
		df_aa_proportion = df_aa_proportion.transpose()
		averaged_aa_control_bar = pd.DataFrame(weighted_contr).fillna(0) # Average AA distribution Bar
		averaged_aa_control_bar = averaged_aa_control_bar.transpose()	# Average AA distribution Bar
		aa_proportion_complete_plot = df_aa_proportion.append(averaged_aa_control_bar)
		####################################
		## Save the reference matrix here ##
		####################################
		with open("Reference_matrix/reference_matrix_%s.pkl" % (frame), "wb") as y:
			pickle.dump(aa_proportion_complete_plot, y)
		with open("Error/ig_seq_residue_error_rate.txt", "w") as fi:
			fi.write("\n")
			fi.write("The residues error rate for this Ig-seq dataset is %f \n" % (residue_error_rate))
		return None

if __name__ == '__main__':

	parser = OptionParser()
	parser.add_option("-s", "--species",
                  action="store", type="string", dest="current_species", default= "human")
	parser.add_option("-c", "--chain",
                  action="store", type="string", dest="chain", default="H")
	(options, args) = parser.parse_args()
	all_aa_proportions(options.chain)
